/*
 *  ad_recorder.mm
 *  AudioDiary
 *
 *  Created by Roy Fejgin on 3/7/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */
#include "AdRecorder.h"
#import "ASIFormDataRequest.h"

AdRecorder::AdRecorder() : m_offset(0)
{
}


void AdRecorder::startRecord()
{
    NSString * directory = NSTemporaryDirectory();    

    // create file name using current time
    NSString* name = [directory stringByAppendingPathComponent:
                                            [NSString stringWithFormat: @"diaryclip-%.0f.%@", 
                                            [NSDate timeIntervalSinceReferenceDate], @"wav"]];

    // open file
	CFURLRef url;
	
	try {		
		// specify the recording format
		setupAudioFormat(kAudioFormatLinearPCM);
        
		url = CFURLCreateWithString(kCFAllocatorDefault, (CFStringRef)name, NULL);
		
		// create the audio file 
		XThrowIfError(AudioFileCreateWithURL(url, kAudioFileWAVEType, &m_recordFormat, kAudioFileFlags_EraseFile,
                                             &m_fileHandle), "AudioFileCreateWithURL failed");

		CFRelease(url);
		
        m_fileName = name;
        [m_fileName retain];
        
	}
	catch (CAXException &e) {
		char buf[256];
		fprintf(stderr, "Error: %s (%s)\n", e.mOperation, e.FormatError(buf));
        NSLog(@"******************Exception!!!!");
	}
	catch (...) {
		fprintf(stderr, "An unknown error occurred\n");
        NSLog(@"*****************Exception!!!!");
	}	
}

// Consider unrolling
void AdRecorder::convertToSint16(SINT16* bufOut, SAMPLE* bufIn, int len)
{
    for (int i=0; i<len; i++) {
        //convert from float to SINT16
        bufOut[i] = (SINT16) (bufIn[i] * 32767.0);
    }
}

ThreadData threaddata = {0};

void* AdRecorder::writeBuf(void* data)
{
    ThreadData* tdata = (ThreadData*)data;
    CircularBuffer<SAMPLE>* buffer = tdata->buf;
    int n = tdata->n;
    AdRecorder* rec = tdata->rec;
    SAMPLE* pFirstChunk;
    SAMPLE* pSecondChunk;
    UInt32 firstChunkLen = buffer->getContiguousChunk(n, n, &pFirstChunk);
    UInt32 firstLen = firstChunkLen;
    UInt32 secondChunkLen = n-firstLen;
    UInt32 secondLen = secondChunkLen;
    NSLog(@"Write buffer: n = %d",n);
    
    buffer->getContiguousChunk(secondChunkLen, secondChunkLen, &pSecondChunk);
    
    UInt32 firstChunkBytes = sizeof(SINT16)*firstChunkLen;
    UInt32 secondChunkBytes = sizeof(SINT16)*secondChunkLen;
    
    SINT16* pTempBuf = new SINT16[MAX(firstChunkLen, secondChunkLen)];
    convertToSint16(pTempBuf, pFirstChunk, firstChunkLen);
    // write first chunk to file
    XThrowIfError(AudioFileWriteBytes(rec->m_fileHandle, FALSE, rec->m_offset,
                                      &firstChunkBytes, pTempBuf),
                  "AudioFileWriteBytes failed");
    // make sure all bytes were written
    assert(firstLen*sizeof(SINT16) == firstChunkBytes);
    rec->m_offset += firstChunkBytes;
    
    if (secondChunkLen > 0) {
        // convrert format
        convertToSint16(pTempBuf, pSecondChunk, secondChunkLen);
        
        // write second chunk to file
        XThrowIfError(AudioFileWriteBytes(rec->m_fileHandle, FALSE, rec->m_offset,
                                          &secondChunkBytes, pTempBuf),
                      "AudioFileWriteBytes failed");
        // make sure all bytes were written
        ASSERT(secondLen*sizeof(SINT16) == secondChunkBytes);
        rec->m_offset += secondChunkBytes;        
    }
    
    //NSLog(@"Write buffer: firstChunk = %d",firstChunkBytes);
    //NSLog(@"Write buffer: secondChunk = %d",secondChunkBytes);
    
    
    delete[] pTempBuf;
    tdata->locked = false;
    return NULL;
}

void AdRecorder::writeBuffer(CircularBuffer<SAMPLE>* buffer, int n, bool sync)
{
    // while this appears non theard-safe, it should work with
    // since this is the only thread that ever attempts to acquire the lock
    while(threaddata.locked) { usleep(500);}
    threaddata.locked = true;
    threaddata.buf = buffer;
    threaddata.n = n;
    threaddata.rec = this;
    
    // Asynchronous write
    if(!sync) {
        //m_thread.setPriority(0);
        m_thread.start(writeBuf,&threaddata);
    }
    // Synchrounous write
    else {
        writeBuf(&threaddata);
    }
}



NSString* AdRecorder::stopRecord()
{
    // close file
    AudioFileClose(m_fileHandle);

    //testFileUpload();
    // return file name
    return m_fileName;
}

void AdRecorder::setupAudioFormat(UInt32 inFormatID)
{
	memset(&m_recordFormat, 0, sizeof(m_recordFormat));
    
	UInt32 size = sizeof(m_recordFormat.mSampleRate);
	XThrowIfError(AudioSessionGetProperty(	kAudioSessionProperty_CurrentHardwareSampleRate,
                                          &size, 
                                          &m_recordFormat.mSampleRate), "couldn't get hardware sample rate");
    // TODO un-hard-code
    m_recordFormat.mSampleRate = 22050;
	m_recordFormat.mChannelsPerFrame = 1;	
	m_recordFormat.mFormatID = inFormatID;
    
	if (inFormatID == kAudioFormatLinearPCM)
	{
		// if we want pcm, default to signed 16-bit little-endian
		m_recordFormat.mFormatFlags = kLinearPCMFormatFlagIsSignedInteger | kLinearPCMFormatFlagIsPacked;
		m_recordFormat.mBitsPerChannel = 16;
		m_recordFormat.mBytesPerPacket = m_recordFormat.mBytesPerFrame = (m_recordFormat.mBitsPerChannel / 8) * m_recordFormat.mChannelsPerFrame;
		m_recordFormat.mFramesPerPacket = 1;
	}
}
