/*
 *  ChordSelector.h
 *  Analyses multi-touch input and computes the selected chord
 *
 *  Created by Roy Fejgin on 2/13/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */
#ifndef __CHORD_SEL__
#define __CHORD_SEL__

#include "mo_touch.h"


typedef enum TriadType
{
    CH_MAJ=0,
    CH_MIN,
    CH_DIM,
    CH_AUG,
    CH_NONE,
    CH_LAST
} TriadType;

typedef enum ExtendedType
{
    EXT_SEV_MAJ=0,
    EXT_SEV_MIN,
    EXT_SEV_DIM,
    EXT_NONE,
    EXT_LAST
} ExtendedType;

// Quadrand data: describes where the touches were
typedef struct QuadrantData
{
    bool quad[4];
    int half[2];
} QuadrantData;

class ChordType
{
public:
    ChordType(): triad(CH_NONE), ext(EXT_NONE) {}
    TriadType triad;
    ExtendedType ext;
    
    // Equality operator
    bool operator == (const ChordType& other) {
        return (other.triad == triad) && (other.ext == ext);
    }

};

class ChordSelector
{
public:    
    ChordSelector(float xSize, float ySize) : m_width(xSize), m_height(ySize){}
    
    bool touchEvent(const std::map<int, MoTouchTrack> & touchPts, UIView * views, ChordType* outChord);
    ChordType getChordType() { return m_chord; }
    QuadrantData getQuadData() { return m_quad; }

private:
    float m_width;
    float m_height;
    ChordType m_chord;
    QuadrantData m_quad;
};





#endif //__CHORD_SEL__