//
//  AudioDiaryAppDelegate.m
//  AudioDiary
//
//  Created by Roy Fejgin on 3/13/10.
//  Copyright __MyCompanyName__ 2010. All rights reserved.
//

#import "AudioDiaryAppDelegate.h"
#import "mo_audio.h"
#import "Stk.h"
#import "mo_gfx.h"



#define SAMPLE_RATE     22050
#define FRAME_SIZE      512
#define NUM_CHANNELS    2



AudioDiaryAppDelegate* g_delegate = NULL;

void audioCallback( Float32 * buffer, UInt32 numFrames, void * userData )
{    
    NSAutoreleasePool * myPool = [[NSAutoreleasePool alloc] init];
    
    for (int i=0; i<numFrames; i++) {
        // Process one input and output sample
        Float32 output = [g_delegate renderAudio: buffer[2*i]];
        
        // Output to all channels
        for (int j=0; j < NUM_CHANNELS; j++) {
            buffer[2*i+j] = output;
        }
    }
    
    [myPool release];
}

@implementation AudioDiaryAppDelegate

@synthesize window;
@synthesize tabBarController;


- (void)applicationDidFinishLaunching:(UIApplication *)application {
    NSArray *timeZones = [NSTimeZone knownTimeZoneNames];

    NSArray* arr = [tabBarController viewControllers];

    // Add the tab bar controller's current view as a subview of the window
    [window addSubview:tabBarController.view];
    
#if 1   
    // init audio
    bool result = MoAudio::init(SAMPLE_RATE, FRAME_SIZE, NUM_CHANNELS);
    if (!result) {
        NSLog(@"Failed to init MoAudio!");
        goto error;
    }
#endif
    // Set up global reference to self
    g_delegate = self;
        
    // Init STK with actual sample rate 
    stk::Stk::setSampleRate( MoAudio::m_hwSampleRate );
#if 1
    // start audio
    result = MoAudio::start(audioCallback, NULL);
    if (!result) {
        NSLog(@"Failed to start MoAudio!");
        goto error;
    }
#endif
    
    [[NSNotificationCenter defaultCenter]
     addObserver:self
     selector:@selector(handleNotification:)
     name:@"DoBrowse"
     object:nil];
    
    // Save references to the three controllers
    m_rec = [arr objectAtIndex:0];
    m_up = [arr objectAtIndex:1];
    m_select = [arr objectAtIndex:2];
    m_browse = [arr objectAtIndex:3];
    
    m_clipManager = new ClipManager();
    [m_rec setClipManager:m_clipManager];
    [m_up setClipManager:m_clipManager];

    [m_select setEvents: &m_events];
    [m_browse setEvents: &m_events];
    
    
	m_up.timeZoneNames = [timeZones sortedArrayUsingSelector:@selector(localizedCaseInsensitiveCompare:)];

    
error:
    return;
}




-(void)handleNotification:(NSNotification *)pNotification
{
    self.tabBarController.selectedViewController = m_browse;
    [[m_browse tableView] reloadData];
    [[m_browse tableView]  scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0] atScrollPosition:UITableViewScrollPositionTop animated:NO];
}
        
        

- (GLfloat) renderAudio: (SAMPLE) input
{
    SAMPLE output = 0.0;
    
    output += [m_rec tick:input];
    output += [m_browse tick:input];
    //output += [up tick:input];
    //output += [browse tick:input];
    
    //[pool release];
    
    return output;
}


/*
// Optional UITabBarControllerDelegate method
- (void)tabBarController:(UITabBarController *)tabBarController didSelectViewController:(UIViewController *)viewController {
}
*/

/*
// Optional UITabBarControllerDelegate method
- (void)tabBarController:(UITabBarController *)tabBarController didEndCustomizingViewControllers:(NSArray *)viewControllers changed:(BOOL)changed {
}
*/


- (void)dealloc {
    [tabBarController release];
    [window release];
    [super dealloc];
}

@end

