//
//  BrowseController.mm
//  AudioDiary
//
//  Created by Roy Fejgin on 3/13/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "BrowseController.h"
#import "my_defs.h"
#import "ASIFormDataRequest.h"

@implementation BrowseController
- (void)awakeFromNib
{
    NSLog(@"Browse Controller awake from Nib");
    m_player = NULL;
}


- (SAMPLE) tick: (SAMPLE) input
{
    if(m_player) {
         if (m_player->isDone()) {
             // restore original label
             [[self tableView ] performSelectorOnMainThread:@selector(reloadData) withObject:nil waitUntilDone:NO];
             
             AudioClipPlayer* acp = m_player;
             m_player = NULL;
             delete acp;
             return 0.0;
         } 
         else {
             return m_player->tick();
         }     
    }
    else {
        return 0.0;
    }
}

/*
- (id)initWithStyle:(UITableViewStyle)style {
    // Override initWithStyle: if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
    if (self = [super initWithStyle:style]) {
    }
    return self;
}
*/

- (void)viewDidLoad {
    [super viewDidLoad];

    NSLog(@"!!!!!!!! Browse Controller view did load !!!!!!!");
    // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
    // self.navigationItem.rightBarButtonItem = self.editButtonItem;
}



- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    // number of rows
    return m_events->size();
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    int row = [indexPath row];
  
    UITableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
    m_cellText = [[cell textLabel]text];
    [m_cellText retain];
    m_playingCell = cell;
    
    [cell textLabel].text = @"Downloading";    
    Event* event= &(*m_events)[row];
    [self downloadFile:event];    
}

- (void) setEvents:(vector<Event> *)events
{
    m_events = events;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	
	static NSString *MyIdentifier = @"MyIdentifier";
	
	// Try to retrieve from the table view a now-unused cell with the given identifier.
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:MyIdentifier];
	
	// If no cell is available, create a new one using the given identifier.
	if (cell == nil) {
		// Use the default cell style.
		cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:MyIdentifier] autorelease];
	}
	int row = indexPath.row;
    Event event = (*m_events)[row];
    
    // Set up the cell.
    
    // Display timestamp
    int ts = [event.num_val intValue];
    NSDate* date = [[[NSDate alloc] initWithTimeIntervalSinceReferenceDate:ts ] autorelease];    
    NSString* desc = [date description];
    NSRange range;
    range.location = 5;
    range.length = [desc length] -4 - 6; // trim 
    cell.textLabel.text = [desc substringWithRange:range];
    
	return cell;
}


extern const NSString * server;
- (void)downloadFile:(Event*) event{
    
    NSLog(@"Trying to download file");
    	
    NSURL * URL = [[NSURL alloc] initWithString:[server stringByAppendingString:@"getFile.php"]];

	// Form the request
	ASIFormDataRequest *request = [ASIFormDataRequest requestWithURL:URL];
	[request setPostValue:event->id forKey:@"myeventid"]; 
	
    NSString *filePath = [NSTemporaryDirectory()
                          stringByAppendingPathComponent:@"dlfile.wav"];

    [request setDownloadDestinationPath:filePath];

	// Start sending synchronously
	[request startSynchronous];
    
    // Display any error messages
	NSError *error = [request error];
	if (error) {
		NSString *response = [request responseString];
        NSLog(@"Error while trying to pull nearby XML");
		NSLog(response);
	}
    else {
        NSLog(@"Got file successfully!!");
    }
    
    if (m_player)  {
        delete m_player;
        m_player = NULL;
    }
    m_player = new AudioClipPlayer([filePath UTF8String], 1., 1);
    m_player->playClip();
    
    [m_playingCell textLabel].text = @"Playing";
    

    [URL release];
}


/*
- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
}
*/

- (void)viewWillDisappear:(BOOL)animated {
	[super viewWillDisappear:animated];
    
    NSLog(@"Browse view disappearing -> deleting audio player");
    if (m_player)  {
        delete m_player;
        m_player = NULL;
    }
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    // Reload table, as new entries may have appeared
    [self performSelectorOnMainThread:@selector(reloadTable) withObject:nil waitUntilDone:NO];
}

-(void) reloadTable {
    [browseView reloadData];
    if ([[self tableView] numberOfRowsInSection:0] > 0) {
        [browseView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0] 
                          atScrollPosition:UITableViewScrollPositionTop animated:NO];
    }
}

/*
- (void)viewDidDisappear:(BOOL)animated {
	[super viewDidDisappear:animated];
}
*/

/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


#pragma mark Table view methods

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/


/*
// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:YES];
    }   
    else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
    }   
}
*/


/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath {
}
*/


/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/


- (void)dealloc {
    [super dealloc];
}


@end

