//
//  UploadController.mm
//  AudioDiary
//
//  Created by Roy Fejgin on 3/13/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "UploadController.h"
#import "ASIFormDataRequest.h"


@implementation UploadController
@synthesize timeZoneNames;
@synthesize queue;
@synthesize m_currentClipLabel;
@synthesize m_indexPath;

- (void)awakeFromNib
{
    NSLog(@"Upload Controller awake from Nib");
    
    
    // Register for notification on recording status (start/stop)
    [[NSNotificationCenter defaultCenter]
     addObserver:self
     selector:@selector(handleNotification:)
     name:@"RecordStatus"
     object:nil];
    
    // set self as delegate to table view
    self.tableView.delegate = self; 
    
    m_uploading = NO;
}


- (void)viewDidLoad {
    NSLog(@"Upload Controller: ViewDidLoad");
;
}

// Update on main thread
-(void) updateDisplay : ( NSString *)str : (UILabel*) label
{
    [label performSelectorOnMainThread : @ selector(setText : ) withObject:str waitUntilDone:YES];
    [label setNeedsDisplay];
    
    [[self tableView] setNeedsDisplay];

}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	// Return the number of time zone names.
	//return [timeZoneNames count];
    return m_clipManager->getClipCount();
}


- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (!m_uploading) {
        [tableView deselectRowAtIndexPath:indexPath animated:NO];

        int row = [indexPath row];        
        NSLog(@"Row %d selected", row);

        UITableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
        
        // Save for async handling 
        m_currentClipLabel = [[cell textLabel] text];
        m_currentClip = row;
        m_indexPath = indexPath;
        m_uploading = YES;
        [self updateDisplay:@"Uploading...":[cell textLabel]];

        // Upload
        [self uploadFile:m_clipManager->elementAt(row)->getFilename():m_clipManager->elementAt(row)];
    }
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	
	static NSString *MyIdentifier = @"MyIdentifier";
	
	// Try to retrieve from the table view a now-unused cell with the given identifier.
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:MyIdentifier];
	
	// If no cell is available, create a new one using the given identifier.
	if (cell == nil) {
		// Use the default cell style.
		cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:MyIdentifier] autorelease];
	}
	
    Clip* clip = m_clipManager->elementAt(indexPath.row);
        
    // date of clip start
    NSDate* date = [[[NSDate alloc] initWithTimeIntervalSinceReferenceDate:clip->getStartTime() ] autorelease];
    
    NSString* desc = [date description];
    NSRange range;
    range.location = 5;
    range.length = [desc length] -4 - 6; // trim 
    cell.textLabel.text = [desc substringWithRange:range];
	
	return cell;
}


- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    // Reload table, as new entries may have appeared
    [self performSelectorOnMainThread:@selector(reloadTable) withObject:nil waitUntilDone:NO];
}

-(void) reloadTable {
    [uploadView reloadData];
    if ([[self tableView] numberOfRowsInSection:0] > 0) {
        [uploadView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0] 
                          atScrollPosition:UITableViewScrollPositionTop animated:NO];
    }
}

-(void)handleNotification:(NSNotification *)pNotification
{
    bool recordStatus = [(NSNumber*)[pNotification object] boolValue];
    
    NSLog(@"UploadController received message %d", recordStatus);
    
    // Recording stopped -> new clip created
    //if (!recordStatus) {
    //    [uploadView reloadData];
    //    [uploadView  scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0] atScrollPosition:UITableViewScrollPositionTop animated:NO];
    //}
}


-(void) setClipManager : ( ClipManager * )mgr
{
    m_clipManager = mgr;
}

- (SAMPLE) tick: (SAMPLE) input
{
    return 0.0;
}

/*
 To conform to Human Interface Guildelines, since selecting a row would have no effect (such as navigation), make sure that rows cannot be selected.
 */
//- (NSIndexPath *)tableView:(UITableView *)tableView willSelectRowAtIndexPath:(NSIndexPath *)indexPath {
//    NSLog(@"willSelectRowAtIndexPath returning nil");
//	return nil;
//}


- (void)dealloc {
	[timeZoneNames release];
	[super dealloc];
}



static ASIFormDataRequest * setupPostVars(ASIFormDataRequest *request, Clip* clip)
{
	// Get the location
	CLLocation * loc = MoLocation::getLocation();
	CLLocationCoordinate2D  coord = [loc coordinate];
	double lat = coord.latitude; 
	double lon = coord.longitude;
	
	NSLog(@"Poll Lat:%f Long:%f", lat, lon);
	
	[request setPostValue:[[UIDevice currentDevice] uniqueIdentifier] forKey:@"device_id"];
	[request setPostValue:[NSString stringWithFormat:@"%f",clip->getCoord().latitude] forKey:@"lat"];
	[request setPostValue:[NSString stringWithFormat:@"%f",clip->getCoord().longitude] forKey:@"long"];
	[request setPostValue:[NSString stringWithFormat:@"%f",clip->getStartTime()] forKey:@"num_val"];
	[request setPostValue:@"Some text" forKey:@"text_val"];	
	
	return request;
	
}


- (void)requestDone:(ASIHTTPRequest *)request
{
    NSLog(@"Request done");
    //NSString *response = [request responseString];

    // Succesful upload ==> delete clip, file and row
    // TODO: delete file!!
    m_clipManager->deleteElementAt(m_currentClip);
    [[self tableView] deleteRowsAtIndexPaths:[NSArray arrayWithObject:m_indexPath] withRowAnimation:UITableViewRowAnimationFade];

    m_uploading = NO;    
}

- (void)requestWentWrong:(ASIHTTPRequest *)request
{

    NSError *error = [request error];    
    NSLog(@"Request went wrong, error %@", [error localizedDescription]);
    
    
    [[self tableView] deselectRowAtIndexPath:m_indexPath animated:NO];
    UITableViewCell *cell = [[self tableView] cellForRowAtIndexPath:m_indexPath];
    [self updateDisplay:m_currentClipLabel:[cell textLabel]];
    
    m_uploading = NO;
}


const NSString * server = @"http://ding.stanford.edu/2010/256b/audiodiary/";

- (BOOL)uploadFile:(NSString*) fileName: (Clip*) clip
{
    BOOL retval = NO;
    
    NSAutoreleasePool	 *autoreleasepool = [[NSAutoreleasePool alloc] init];
    
	NSLog(@"Uploading %@", fileName);
	
    if (![self queue]) {
        [self setQueue:[[[NSOperationQueue alloc] init] autorelease]];
    }
    
    NSURL *url = [NSURL URLWithString:[server stringByAppendingString:@"saveMyFile.php"]];
	
    ASIFormDataRequest *request = [ASIFormDataRequest requestWithURL:url];
	
	// Setup the file for the request
	[request setFile:fileName forKey:@"uploaded"];
	
	// Setup variables for the request (location, device ID)
	request = setupPostVars(request, clip);
	
    [request setDelegate:self];
    [request setDidFinishSelector:@selector(requestDone:)];
    [request setDidFailSelector:@selector(requestWentWrong:)];
    
    
	// Start sending synchronously
	//[request startAsynchronous];
    NSLog(@"***** Starting Upload *****");
    //[request startAsynchronous];
    [[self queue] addOperation:request];
    //[request startSynchronous];
    NSLog(@"***** After Upload Request *****");
	
	// Display any error messages
	NSError *error = [request error];
	if (!error) {
		//NSString *response = [request responseString];
		NSLog(@"Got response from server");//response);//[text setText:response];
        retval = YES;
	} else {
        NSLog(@"HTTP error!! :(");
    }
    // do thread work
    [autoreleasepool release];
    
    return retval;
}


@end

