/*
 *  ClipManager.h
 *  AudioDiary
 *
 *  Created by Roy Fejgin on 3/15/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */


#ifndef __CLIP_MGR_H__
#define __CLIP_MGR_H__

#import <list>
#import <vector>

#import "Clip.h"
#import "AudioClipPlayer.h"
#import "mo_location.h"

using namespace std;

class ClipManager 
{
public:
    
    // constructor
    ClipManager();
    
    ~ClipManager();
    
    // add a clip to the database
    void addClip(CLLocation* location,
                 NSTimeInterval startTime,
                 float length,
                 NSString* fileName,
                 NSString* userName = NULL,
                 NSString* tag = NULL);
    
    void addClip(Clip* clip);
    
    // Allows user to iterate over all clips
    // Removing or adding clips to the clip manager invalidates the iteration,
    // so in that case the user should call startIterating() again after the 
    // modification.
    void startIteration();
    
    int getClipCount() { return m_clips.size(); };
    Clip* elementAt(int i) { return &m_clips[i]; }
    void deleteElementAt(int i) { m_clips.erase(m_clips.begin()+i);};
    
    Clip* nextClip();
    
    // remove from database and delete associated wav file
    void deleteClip(Clip* clip);
    
    // play clip
    void playClip(Clip* clip);
    
    // audio tick
    SAMPLE tick();
    
    // DEBUG
    
    void listDirectory();
    
    
private:
    vector<Clip> m_clips;
    list<Clip>::iterator m_clipIter;
    
    //list<AudioClipPlayer*> m_players;
    AudioClipPlayer* m_player;

};



#endif // __CLIP_MGR_H__