//
//  ES1Renderer.m
//  
//
//  Created by Roy Fejgin on 1/29/10.
//  Copyright __MyCompanyName__ 2010. All rights reserved.
//

#import "ES1Renderer.h"

#import "Texture2D.h"

// Mopho API
#include "mo_gfx.h"
#include "mo_audio.h"
#include "mo_touch.h"
#include "mo_audio.h"
// STK
#include "Stk.h"

// STL
#include <list>

// project includes
#include "my_defs.h"
#include "my_util.h"

using namespace std;

/*********************************************************************************
  Macros
 **********************************************************************************/

// Macros
#define SAMPLE_RATE    16000
#define FRAME_SIZE     512
#define NUM_CHANNELS   2


/*********************************************************************************
 Globals
 **********************************************************************************/

// Note names
const NSString* g_notes[12] = {@"A", @"Bb", @"B", @"C", @"C#", @"D", @"Eb", @"E", @"F",@"F#", @"G",@"Ab"};

// Texture name (handle)
GLuint g_triadTextures[CH_LAST] = {0};
GLuint g_extTextures[EXT_LAST] = {0};
GLuint g_combinedTextures[CH_LAST][EXT_LAST] = {0};
GLuint g_noteNames[12] = {0};

ES1Renderer* g_view = NULL;


/*********************************************************************************
  Function Implementations
 **********************************************************************************/

// Touch callback
void touchCallback( NSSet * touches, UIView * view, 
                   const std::map<int, MoTouchTrack> * touchPts,
                   void * data)
{
    // Forward the call to the main view
    [g_view touchEvent: touchPts : view];
}


// Audio callback
void audioCallback( Float32 * buffer, UInt32 numFrames, void * userData )
{    
    for (int i=0; i<numFrames; i++) {
        // Get sample from the world simulation
        Float32 output = [g_view renderAudio: buffer[2*i]];

        // Output to all channels
        for (int j=0; j < NUM_CHANNELS; j++) {
            buffer[2*i+j] = output;
        }
    }
}

/*********************************************************************************
 ES1Renderer implementation
 **********************************************************************************/

@implementation ES1Renderer

// Create an ES 1.1 context
- (id) init
{
	if (self = [super init])
	{
		context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES1];
        
        if (!context || ![EAGLContext setCurrentContext:context])
		{
            [self release];
            return nil;
        }
		
		// Create default framebuffer object. The backing will be allocated for the current layer in -resizeFromLayer
		glGenFramebuffersOES(1, &defaultFramebuffer);
		glGenRenderbuffersOES(1, &colorRenderbuffer);
		glBindFramebufferOES(GL_FRAMEBUFFER_OES, defaultFramebuffer);
		glBindRenderbufferOES(GL_RENDERBUFFER_OES, colorRenderbuffer);
		glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_COLOR_ATTACHMENT0_OES, GL_RENDERBUFFER_OES, colorRenderbuffer);
	}
    
    g_view = self;
    
    // init audio
    bool result = MoAudio::init(SAMPLE_RATE, FRAME_SIZE, NUM_CHANNELS);
    if (!result) {
        NSLog(@"Failed to init MoAudio!");
        return nil;
    }

    // Create main objects
    pModel = new HarmonyModel(FRAME_SIZE, MoAudio::m_hwSampleRate);
    pUIChord = new UI_Chord(pModel);
    
    // Load textures
    [self loadTextures];
    
    // Init STK with actual sample rate 
    stk::Stk::setSampleRate( MoAudio::m_hwSampleRate );
    
    // start audio
    result = MoAudio::start(audioCallback, NULL);
    if (!result) {
        NSLog(@"Failed to start MoAudio!");
        return nil;
    }
    
    // Register touch callback
    MoTouch::addCallback(touchCallback, self);
    
    
    [[NSNotificationCenter defaultCenter] addObserver: self selector:
     @selector(fluteOn) name: @"FluteOn" object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver: self selector:
     @selector(fluteOff) name: @"FluteOff" object:nil];
 	return self;   
}

- (void)fluteOff
{
    NSLog(@"Got flute off");
    pModel->setFluteOn(false);
}

- (void)fluteOn
{
    NSLog(@"Got flute on");
    pModel->setFluteOn(true);

}

    



- (void) render
{
    // Replace the implementation of this method to do your own custom drawing
    
	// This application only creates a single context which is already set current at this point.
	// This call is redundant, but needed if dealing with multiple contexts.
    [EAGLContext setCurrentContext:context];
    
	// This application only creates a single default framebuffer which is already bound at this point.
	// This call is redundant, but needed if dealing with multiple framebuffers.
    glBindFramebufferOES(GL_FRAMEBUFFER_OES, defaultFramebuffer);
    glViewport(0, 0, backingWidth, backingHeight);
    
    // Set orthographic projection
    // Note: I've modified the ortho() function to compensate for 
    // assymetric aspect ratio of the iPhone's screen. 
    MoGfx::ortho();
    
    glMatrixMode(GL_MODELVIEW);    
	glLoadIdentity();
	
    glClearColor(0.5f, 0.5f, 0.5f, 1.0f);
    glClear(GL_COLOR_BUFFER_BIT);

    // Draw chord-selecting interface
    pUIChord->draw();
    
	// This application only creates a single color renderbuffer which is already bound at this point.
	// This call is redundant, but needed if dealing with multiple renderbuffers.
    glBindRenderbufferOES(GL_RENDERBUFFER_OES, colorRenderbuffer);
    [context presentRenderbuffer:GL_RENDERBUFFER_OES];
}
- (GLfloat) renderAudio: (SAMPLE) input
{
    return pModel->tick(input);
}

- (void) touchEvent:(const std::map<int, MoTouchTrack>*) touchPts: (UIView*) view
{
    pModel->touchEvent(*touchPts, view);
}

- (BOOL) resizeFromLayer:(CAEAGLLayer *)layer
{	
	// Allocate color buffer backing based on the current layer size
    glBindRenderbufferOES(GL_RENDERBUFFER_OES, colorRenderbuffer);
    [context renderbufferStorage:GL_RENDERBUFFER_OES fromDrawable:layer];
	glGetRenderbufferParameterivOES(GL_RENDERBUFFER_OES, GL_RENDERBUFFER_WIDTH_OES, &backingWidth);
    glGetRenderbufferParameterivOES(GL_RENDERBUFFER_OES, GL_RENDERBUFFER_HEIGHT_OES, &backingHeight);
	
    if (glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES) != GL_FRAMEBUFFER_COMPLETE_OES)
	{
		NSLog(@"Failed to make complete framebuffer object %x", glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES));
        return NO;
    }
    
    return YES;
}

- (void) dealloc
{
	// Tear down GL
	if (defaultFramebuffer)
	{
		glDeleteFramebuffersOES(1, &defaultFramebuffer);
		defaultFramebuffer = 0;
	}

	if (colorRenderbuffer)
	{
		glDeleteRenderbuffersOES(1, &colorRenderbuffer);
		colorRenderbuffer = 0;
	}
	
	// Tear down context
	if ([EAGLContext currentContext] == context)
        [EAGLContext setCurrentContext:nil];
	
	[context release];
	context = nil;
	
	[super dealloc];
}
- (void) loadTextTexture:(NSString*) pStr:(GLuint*) name:(BOOL)small;
{
    int width = 100, height = 100;
    int fSize = 35;
    UITextAlignment al = UITextAlignmentCenter;
    NSString* fName = @"Times New Roman";
    if(small) {
        fName = @"AmericanTypewriter-Bold";
        width = 200;
        fSize = 25;
        al = UITextAlignmentLeft;
    }
    
    Texture2D* texture = [[Texture2D alloc] initWithString:pStr dimensions:CGSizeMake(width, height)
                                                 alignment:al fontName:fName fontSize:fSize];
    
    *name = [texture name];
    // Bind texture
    glBindTexture(GL_TEXTURE_2D, *name);
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
}

- (void) loadTextTexture:(NSString*) pStr: (GLuint*) name
{
    [self loadTextTexture:pStr:name:false];
}


// Load textures for the entire program
- (void) loadTextures
{
    // enable texture mapping
    glEnable( GL_TEXTURE_2D );
    // enable blending
    glEnable( GL_BLEND );
    // blend function
    // glBlendFunc( GL_ONE, GL_ZERO );
    glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
    
    // Triads
    [self loadTextTexture:@"Major":&g_triadTextures[0]];
    [self loadTextTexture:@"minor":&g_triadTextures[1]];
    [self loadTextTexture:@"dim":&g_triadTextures[2]];
    [self loadTextTexture:@"Aug":&g_triadTextures[3]];
    
    // Seventh
    [self loadTextTexture:@"Maj7":&g_extTextures[0]];
    [self loadTextTexture:@"m7":&g_extTextures[1]];
    [self loadTextTexture:@"dim7":&g_extTextures[2]];
    
    // Combined
    [self loadTextTexture:@"Maj7":&g_combinedTextures[CH_MAJ][EXT_SEV_MAJ]:true];
    [self loadTextTexture:@"m Maj7":&g_combinedTextures[CH_MIN][EXT_SEV_MAJ]:true];
    [self loadTextTexture:@"dim Maj7":&g_combinedTextures[CH_DIM][EXT_SEV_MAJ]:true];
    [self loadTextTexture:@"+ Maj7":&g_combinedTextures[CH_AUG][EXT_SEV_MAJ]:true];

    [self loadTextTexture:@"7":&g_combinedTextures[CH_MAJ][EXT_SEV_MIN]:true];
    [self loadTextTexture:@"m7":&g_combinedTextures[CH_MIN][EXT_SEV_MIN]:true];
    [self loadTextTexture:@"dim 7":&g_combinedTextures[CH_DIM][EXT_SEV_MIN]:true];
    [self loadTextTexture:@"+7":&g_combinedTextures[CH_AUG][EXT_SEV_MIN]:true];

    [self loadTextTexture:@"dim7":&g_combinedTextures[CH_MAJ][EXT_SEV_DIM]:true];
    [self loadTextTexture:@"m dim7":&g_combinedTextures[CH_MIN][EXT_SEV_DIM]:true];
    [self loadTextTexture:@"dim dim7":&g_combinedTextures[CH_DIM][EXT_SEV_DIM]:true];
    [self loadTextTexture:@"+ dim7":&g_combinedTextures[CH_AUG][EXT_SEV_DIM]:true];
    
    [self loadTextTexture:@"":&g_combinedTextures[CH_MAJ][EXT_NONE]:true];
    [self loadTextTexture:@"m":&g_combinedTextures[CH_MIN][EXT_NONE]:true];
    [self loadTextTexture:@"dim":&g_combinedTextures[CH_DIM][EXT_NONE]:true];
    [self loadTextTexture:@"+":&g_combinedTextures[CH_AUG][EXT_NONE]:true];
    
    // Note names
    for (int i=0; i<12; i++) {
        [self loadTextTexture:g_notes[i]:&g_noteNames[i]:true];
    }

}

@end
