/*
 *  HarmonyAudio.h
 *  Generates audio according to the requested harmony.
 *
 *  The harmony can be generated by:
 *  - pitch shifting the input audio, or
 *  - [not implemented:] synthesizing the harmony audio using an STK instrument.
 *
 *  This class internally manages up to five concurrent voices.
 *
 *  Created by Roy Fejgin on 2/14/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#ifndef __HARMONY_AUDIO__
#define __HARMONY_AUDIO__

#import "mo_audio.h"
#import "PitShift.h"
#import "LentPitShift.h"
#import "VoicForm.h"
#import "Flute.h"
#import "BlowHole.h"
#import <list>
#import <vector>

using namespace stk;


class HarmonyAudio
{
public:
    HarmonyAudio();
    
    SAMPLE tick( SAMPLE input);
    
    void setHarmony(float fundFreq, vector<float>& ratios, bool copyRatios = true);
    void setFundFreq(float fundFreq) { setHarmony(fundFreq, m_ratios, false); }
    void clear();
    
    void setMode(bool fluteOn) { m_isPitShiftMode = !fluteOn;}
    inline bool isFluteOn() { return !m_isPitShiftMode; }
    
    static const int MAX_VOICES = 5;

private:
    PitShift * m_shifters[MAX_VOICES];
    Flute * m_voices[MAX_VOICES];
    //VoicForm * m_voices[MAX_VOICES];
    vector<float> m_ratios;
    bool m_active[MAX_VOICES];
    bool m_isPitShiftMode;
};



#endif //__HARMONY_AUDIO__