/*
 *  HarmonyModel.h
 *  Harmonizer
 *
 *  Created by Roy Fejgin on 2/13/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#ifndef __HARMONY_MODEL__
#define __HARMONY_MODEL__

#import "mo_touch.h"
#import "ChordSelector.h"
#import "HarmonyAudio.h"
#import "PitchDetector.h"
#import "my_adsr.h"
#import "my_defs.h"

#define NOTE_RANGE  64
#define NOTE_INVALID   (-1)

class HarmonyModel
{
public:
    HarmonyModel(int frameSize, float sampleRate);

    // Touch callback
    void touchEvent(const std::map<int, MoTouchTrack> & touchPts, UIView * view);
    
    ChordType getChordType() { return m_chordSelector->getChordType(); }
    QuadrantData getQuadData() { return m_chordSelector->getQuadData(); }
    int getNote() {
        if (m_pitchValid) {
            return m_currentNote;
        }
        else {
            return NOTE_INVALID;
        }
    }
    
    SAMPLE tick(SAMPLE input);
    void setFluteOn(bool on) {
        for(int i=0; i< HE_LAST; i++) {
            m_harmonyAudio[i]->setMode(on);}
    }


private:
    // 2 ^ (1/12)
    static const double semiToneFactor = 1.0594630943592953;
    static const double A2Freq = 110.0;
    typedef enum HarmonyElement {
        HE_TRIAD = 0,
        HE_EXT,
        HE_LAST
    } HarmonyElement;
    
    // Generate chord: returns a set of ratios between the notes in the chord
    // and the fundamental frequency
    void buildTriad(ChordType cType, vector<float>& ratios);
    void buildExt(ChordType cType, vector<float>& ratios);
    
    void initFreqMapping();
    int lookupFreq(float freq, float* outFreq);
    
    float degToRatio(int deg) {
        return pow(semiToneFactor, deg);
    }
    
    // Analyeses touches and computes chord
    ChordSelector* m_chordSelector;
    
    // Chord audio generators
    HarmonyAudio* m_harmonyAudio[HE_LAST];
    int m_harmonyIndex;

    // ADSRs for smoothly switching between chords
    MyADSR* m_adsr[HE_LAST];
    
    OnePole* m_onePole;
    
    // Pitch detector (disabled)
    PitchDetector* m_pitchDetector;
    // Pitch is only valid when the input signal has enough energy 
    // for pitch detection
    bool m_pitchValid;
    
    float m_freqs[NOTE_RANGE];
    
    int m_currentNote;
    float m_currentFreq;
    float m_targetFreq;
    OsMutex m_mutex;
        
};

#endif //__HARMONY_MODEL__
