/*
 *  TapAnalyzer.cpp
 *  AudioDiary
 *
 *  Created by Roy Fejgin on 3/8/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#include "TapAnalyzer.h"
#include "mo_audio.h"


#define TAP_THRESH (0.01)
#define GESTURE_TAPS  4
#define BACKOFF_MS  40

TapAnalyzer::TapAnalyzer(CircularBuffer<SAMPLE>& buffer, int maxLookback) :
m_buffer(buffer), m_maxLookback(maxLookback) {
    
    m_onePole = new OnePole(0.99);
    m_backoff = 0;
}

TapType TapAnalyzer::tick(SAMPLE input)
{
    static int tapCount = 0;
    //static in sampsSinceLastTap = 0;
    static 	bool recording = false;
    TapType tapType = TAP_NONE;
    SAMPLE filterVal = m_onePole->tick(input*input);
    
    if (m_backoff == 0) {
        if (filterVal > TAP_THRESH) {
            NSLog(@"tap detected");
            tapCount++;
            // hold off for 20ms
            m_backoff = MoAudio::m_hwSampleRate * (BACKOFF_MS*0.001);
        }
        
        if (tapCount > GESTURE_TAPS) {
            recording = !recording;
            NSLog(@"Gesture detected, recording = %d", recording);
            tapCount = 0;
            if (recording) {
                tapType = TAP_START;
            } else {
                tapType = TAP_STOP;
            }

        }
    } 
    else {
        m_backoff--;
    }
    
    return tapType;
 /*   
    counter++;
    
    if(counter == 44100/2) {
        return TAP_START;
    }
    else if (counter == 44100) {
        return TAP_STOP;
    }
    else {
        return TAP_NONE;
    }
   */
}