/*
 *  adsr.h
 *
 *  An ADSR envelope generator
 *
 *  Created by Roy Fejgin on 1/13/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#ifndef __MY_ADSR
#define __MY_ADSR

class MyADSR {
public:
    MyADSR(int attackSamps, int decaySamps, Float32 sustainLevel, int releaseSamps, bool autoRelease=true) {
        m_level = 0.0;
        m_state = ST_OFF;
        m_sampleNum = 0;
        m_autoRel = autoRelease;
        m_releaseSamps = releaseSamps;
        m_attackSamps = attackSamps;
        
        m_endSample[ST_ATTACK] = attackSamps;
        m_step[ST_ATTACK] = 1.0 / (Float32)attackSamps;
        
        m_endSample[ST_DECAY] = attackSamps+decaySamps;
        m_step[ST_DECAY] = (sustainLevel-1.0)/decaySamps;
        
        if(!autoRelease) {
            m_endSample[ST_SUSTAIN] = INT_MAX;
        }
        else {
            m_endSample[ST_SUSTAIN] = m_endSample[ST_DECAY];
        }
        m_step[ST_SUSTAIN] = 0;
        
        m_endSample[ST_RELEASE] = INT_MAX;
        m_step[ST_RELEASE] = -sustainLevel/(Float32)releaseSamps;
        
        m_endSample[ST_OFF] = INT_MAX;
        m_step[ST_OFF] = 0.0;
    }
    
    void noteOn()
    { 
        m_step[ST_ATTACK] = (1.0-m_level)/m_attackSamps;
        m_state = ST_ATTACK;
        //m_level = 0.0;
        m_sampleNum = 0;
        m_endSample[ST_RELEASE] = INT_MAX;
    }
    
    void noteOff() {
        if (m_state < ST_RELEASE) {
            m_state = ST_RELEASE;
            m_endSample[ST_RELEASE] = m_sampleNum + m_releaseSamps;
            m_step[ST_RELEASE] = (-m_level)/m_releaseSamps;
        }
    }
    
    void setAutoRel(bool enable) { m_autoRel = enable; }
    bool isNoteDone() { return m_state == ST_OFF; }
    
    Float32 tick(Float32 input)
    {
        Float32 level = m_level;
        m_level += m_step[m_state];
        if (m_level < 0) {
            m_level = 0;
        }

        m_sampleNum++;
        
        // Advance state
        if(m_sampleNum > m_endSample[(int)m_state]) {
            m_state = (EnvState)((int)m_state+1);
            //NSLog(@"Changing env state to %d",(int)m_state);
        }
        
        return level;
    }
    
private:
    // Envelope state
    typedef enum EnvState {
        ST_ATTACK = 0,
        ST_DECAY,
        ST_SUSTAIN,
        ST_RELEASE,
        ST_OFF,
        ST_LAST
    } EnvState;
    
    int m_endSample[ST_LAST];
    Float32 m_step[ST_LAST];
    Float32 m_level;
    int m_sampleNum;
    int m_releaseSamps;
    int m_attackSamps;
    EnvState m_state;
    bool m_autoRel;
};

#endif // __MY_ADSR
