/*
 *  my_util.h
 *  
 *
 *  Created by Roy Fejgin on 1/31/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#ifndef __MY_UTIL__
#define __MY_UTIL__

#import "mo_gfx.h"
#import "my_defs.h"
#include "mo_fft.h"

inline CGPoint converToWorld(CGPoint input)
{
    GLfloat x = input.x;
    GLfloat y = input.y;
    
    input.x = -1.+2.*x/320.;
    input.y = -1.5+3.*(1.-y/480.);
//    input.y = -1.+2.*(1.-y/320.);
    //input.y = -1+2.*(1-((y/480.)*480./320.));
    
    return input;
}

inline bool isPointInScreen(Float32 x, Float32 y, Float32 radius)
{
    return !((x+radius<-1) || (x-radius>1) || (y+radius<-1.5) || (y-radius>1.5));
}

// Leaky integrate
inline Float32 leakyIntegrate(Float32 targetVal, Float32 currVal, Float32 factor)
{
    return (factor*currVal) + ((1-factor)*targetVal);
}


// Compute distance between two points
inline Float32 computeDistance(CGPoint p1, CGPoint p2)
{
    Float32 x1,y1,x2,y2;
    x1 = p1.x;
    y1 = p1.y;
    x2 = p2.x;
    y2 = p2.y;
    
    return sqrt(pow(x1-x2,2)+pow(y1-y2,2));
}

// Compute distance between two points
inline Float32 computeDistance3D(Vector3D p1, Vector3D p2)
{
    Float32 x1,y1,z1,x2,y2,z2;
    x1 = p1.x;
    y1 = p1.y;
    z1 = p1.z;
    x2 = p2.x;
    y2 = p2.y;
    z2 = p2.z;
    
    return sqrt(pow(x1-x2,2)+pow(y1-y2,2) + pow(z1-z2, 2));
}

inline Vector3D project(Vector3D src, Vector3D dst)
{
    dst.normalize();
    Float32 dot = src*dst;
    dst *= dot;
    return dst;
}

// This only works for 2D !!!
inline Vector3D orthoVec(Vector3D src)
{
    Vector3D result;
    result.x = -src.y;
    result.y = src.x;
    result.z = 0;
    return result;
}


inline complex complexMultiply(complex * a, complex*b)
{
    complex result;
    result.re = a->re * b->re - a->im * b->im;
    result.im = a->re * b->im + a->im * b->re;
    return result;
}

inline int findMaxIndex(float* pBuffer, int n)
{
    float max = pBuffer[0];
    int maxBin = 0;
    for(int i=0; i<n; i++)
    {
        if(pBuffer[i] > max) {
            max = pBuffer[i];
            maxBin = i;
        }
    }
    return maxBin;
}        


inline int findFirstPeak(float* pBuffer, int n)
{
    float prev = pBuffer[0];
    int maxIndex = 0;
    bool rising = false;
    
    for(int i=1; i<n; i++)
    {
        // Detect transition
        if(rising && (pBuffer[i] < prev)) {
            maxIndex =  i;
            break;
        }
        if(pBuffer[i] > prev) {
            rising = true;
        }
        
        prev = pBuffer[i];
    }
    return maxIndex;
}



#ifdef DEBUG_ON || 1
#define ASSERT(_x)   my_assert(_x)
inline void my_assert(bool cond)
{
    // the whole point of this function is to allow placing a breakpoint on 
    // failed asserts
    if (!cond) {
        assert(0);
    }
}
#else
#define ASSERT(_x)   assert(_x)
#endif


#endif __MY_UTIL__
