/*
 *  ui_chord.mm
 *  Harmonizer
 *
 *  Created by Roy Fejgin on 2/14/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#include "ui_chord.h"
#include "mo_gfx.h"

#import "Texture2D.h"

void drawText(GLuint* handle, float x, float y, float scale=0.5, GLubyte* pColor = NULL)
{
    GLfloat factor = scale;    
    GLfloat squareVertices[] = {
        factor*-1.0f, factor*-1.0f,
        factor*1.0f,  factor*-1.0f,
        factor*-1.0f, factor*1.0f,
        factor*1.0f,  factor*1.0f
    };

    static const GLubyte squareColors[] = {
        161, 13, 0x97, 255,
        161, 13, 0x97, 255,
        161, 13, 0x97, 255,
        161, 13, 0x97, 255,
    };
    
    
    static const GLfloat texCoords[] = {
        0, 1,
        1, 1,
        0, 0,
        1, 0
    };
    
    glBindTexture(GL_TEXTURE_2D, *handle);

    
    glPushMatrix();
    
    // Move text to the right place
    glTranslatef(x,y,0);
    
    if (pColor != NULL) {
        glColorPointer(4, GL_UNSIGNED_BYTE, 0, pColor); 
    }
    else {
        glColorPointer(4, GL_UNSIGNED_BYTE, 0, squareColors);
    }


    glEnableClientState(GL_COLOR_ARRAY);
    glVertexPointer(2, GL_FLOAT, 0, squareVertices);
    glEnableClientState(GL_VERTEX_ARRAY);
    
    // texture coordinate
    glTexCoordPointer( 2, GL_FLOAT, 0, texCoords );
    glEnableClientState( GL_TEXTURE_COORD_ARRAY );

    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

    glPopMatrix();
}

void UI_Chord::draw()
{    
    ChordType chord = m_model->getChordType();
    QuadrantData quad = m_model->getQuadData();
    drawQuads(quad);
    drawSeparators(quad);
    drawLabels(quad,chord);    
    drawActiveChord(chord);
}

void UI_Chord::drawLabels(QuadrantData& quad, ChordType chord)
{
    if (quad.half[1] <=1) {
        drawText(&g_triadTextures[CH_MAJ], -0.4,-1);
        drawText(&g_triadTextures[CH_MIN], -0.4,.5);
    }
    else {
        drawText(&g_triadTextures[chord.triad], -0.4,-1);
        drawText(&g_triadTextures[chord.triad], -0.4,.5);
    }
    if (quad.half[0] <=1) {
        drawText(&g_extTextures[EXT_SEV_MAJ], .55, -1);
        drawText(&g_extTextures[EXT_SEV_MIN], .55, .5);
    }
    else {
        drawText(&g_extTextures[chord.ext], .55, -1);
        drawText(&g_extTextures[chord.ext], .55, .5);
    }
}

void UI_Chord::drawActiveChord(ChordType chord)
{
    // Draw active chord near bottom of screen
    if (chord.triad != CH_NONE) {
        int note = m_model->getNote();
        assert(note<12);
        
        // Disable texture mapping
        glDisable(GL_TEXTURE_2D);
        glDisable(GL_BLEND);
        
        GLfloat squareVertices[] = {
            -0.72, -.2,
            0.72, -.2,
            -0.72, .2,
            0.72, .2
        };

        static GLubyte red[] = {
            255, 0, 0, 255,
            255, 0, 0, 255,
            255, 0, 0, 255,
            255, 0, 0, 255
        };
        
        // How much to translate
        glColorPointer(4, GL_UNSIGNED_BYTE, 0, red);
        glEnableClientState(GL_COLOR_ARRAY);
        glVertexPointer(2, GL_FLOAT, 0, squareVertices);
        glEnableClientState(GL_VERTEX_ARRAY);
        
        glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);   
        
        // Re-enable texture mapping
        glEnable(GL_BLEND);
        glEnable(GL_TEXTURE_2D);
        
        // yellow
        static GLubyte yellow[] = {
            255, 0, 0, 255,
            255, 0, 0, 255,
            200, 255, 0, 255,
            200, 255, 0, 255,
        };
        
        drawText(&g_combinedTextures[chord.triad][chord.ext], 0.76, -.75 , 1, yellow);
        if (note != NOTE_INVALID) {
            drawText(&g_noteNames[note], 0.5, -.75 , 1, yellow);  
        }

    }
}
    
void UI_Chord::drawQuads(const QuadrantData& quad)
{
    // Disable texture mapping
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_BLEND);
    
    
    GLfloat squareVertices[] = {
        -1, -1.5,
        0, -1.5,
        -1, 0,
        0,  0
    };

    
    static GLubyte aqua[] = {
        117, 255, 221, 255,
        117, 255, 221, 255,
        117, 255, 221, 255,
        117, 255, 221, 255
    };
    static GLubyte blue[] = {
        0x18,0x5e,0xf5,255,
        0x18,0x5e,0xf5,255,
        0x18,0x5e,0xf5,255,
        0x18,0x5e,0xf5,255
    };
    
    static GLubyte white[] = {
        255, 255, 255, 255,
        255, 255, 255, 255,
        255, 255, 255, 255,
        255, 255, 255, 255
    };
    
    static GLubyte orange[] = {
        0xf0, 0x5f, 0x11, 255,
        0xf0, 0x5f, 0x11, 255,
        0xf0, 0x5f, 0x11, 255,
        0xf0, 0x5f, 0x11, 255
    };
    
    // How much to translate
    const float xOffsets[] = {1,0,0,1};
    const float yOffsets[] = {1.5,1.5,0,0};
    const int indexToHalf[] = {0,1,1,0};
    
    for (int i=0; i<4; i++) {

        // Select color
        GLubyte* pColor = NULL;
        switch(quad.half[indexToHalf[i]]) {
            case 1:
                if (quad.quad[i]) {
                    pColor = white;
                }
                break;
            case 2:
                if(indexToHalf[i] == 0) {
                    pColor = blue;
                }
                else {
                    pColor = aqua;
                }
                break;
            case 3: pColor = orange;
                break;
            default:
                break;
                
        }
            
        if (pColor != NULL) {
            
            // Push
            glPushMatrix();
            
            // Move it to the right place
            glTranslatef(xOffsets[i], yOffsets[i] ,0);
            
            glColorPointer(4, GL_UNSIGNED_BYTE, 0, pColor);
            glEnableClientState(GL_COLOR_ARRAY);
            glVertexPointer(2, GL_FLOAT, 0, squareVertices);
            glEnableClientState(GL_VERTEX_ARRAY);
            
            glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);   
            
            // Pop
            glPopMatrix();
        }
    }
    
    // Re-enable texture mapping
    glEnable(GL_BLEND);
    glEnable(GL_TEXTURE_2D);
}


void UI_Chord::drawSeparators(QuadrantData& quad)
{
    // Disable texture mapping
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_BLEND);

    GLfloat vertLine[] = {
        0, 1.5,
        0, -1.5
    };
    GLfloat horzLine[] = {
        -1, 0,
        1, 0
    };
    
    
    GLfloat frame[] = {
        -.95,-1.5,
        -.95,1.5,
        -.95,1.45,
        .95,1.45,
        .96,1.5,
        .96,-1.5,
        .95,-1.46,
        -.95,-1.46,
    };
    
    GLubyte col[] = {
        0xa5,0x87,0xff,255,
        0xa5,0x87,0xff,255,
        0xa5,0x87,0xff,255,
        0xa5,0x87,0xff,255,
        0xa5,0x87,0xff,255,
        0xa5,0x87,0xff,255,
        0xa5,0x87,0xff,255,
        0xa5,0x87,0xff,255,
    };
    
    glPushMatrix();
    
    glColorPointer(4, GL_UNSIGNED_BYTE, 0, col);
    glEnableClientState(GL_COLOR_ARRAY);
    
    // Vertical line
    glVertexPointer(2, GL_FLOAT, 0, vertLine);
    glEnableClientState(GL_VERTEX_ARRAY);
    
    glLineWidth(15.);
    glDrawArrays(GL_LINES, 0, 2);

    glColorPointer(4, GL_UNSIGNED_BYTE, 0, col);
    glEnableClientState(GL_COLOR_ARRAY);
    
    // Horizontal line
    glVertexPointer(2, GL_FLOAT, 0, horzLine);
    glEnableClientState(GL_VERTEX_ARRAY);
    glDrawArrays(GL_LINES, 0, 2);

    // Frame
    glColorPointer(4, GL_UNSIGNED_BYTE, 0, col);
    glEnableClientState(GL_COLOR_ARRAY);
    glVertexPointer(2, GL_FLOAT, 0, frame);
    glEnableClientState(GL_VERTEX_ARRAY);
    glDrawArrays(GL_LINES, 0, 8);    
    
    glPopMatrix();
    
    // Re-enable texture mapping
    glEnable(GL_BLEND);
    glEnable(GL_TEXTURE_2D);
}
    